@@ -26,23 +26,23 @@
 import argparse
 import logging
 import os
-from requests.exceptions import ConnectionError
 import sys
 import time
 
-from ceilometerclient.v2 import client as ceilometer_client
 import ceilometerclient.exc
+from ceilometerclient.v2 import client as ceilometer_client
 import cinderclient.exceptions
 from cinderclient.v1 import client as cinder_client
-from glanceclient.v1 import client as glance_client
 import glanceclient.exc
+from glanceclient.v1 import client as glance_client
 from keystoneclient.apiclient import exceptions as api_exceptions
-from keystoneclient.v2_0 import client as keystone_client
 import keystoneclient.openstack.common.apiclient.exceptions
+from keystoneclient.v2_0 import client as keystone_client
 import neutronclient.common.exceptions
 from neutronclient.v2_0 import client as neutron_client
 import novaclient.exceptions
 from novaclient.v1_1 import client as nova_client
+import requests
 from swiftclient import client as swift_client
 
 RETRIES = 10  # Retry a delete operation 10 times before exiting
@@ -78,6 +78,7 @@ NOT_AUTHORIZED = 6
 # Available resources classes
 
 RESOURCES_CLASSES = ['CinderSnapshots',
+                     'CinderBackups',
                      'NovaServers',
                      'NeutronFloatingIps',
                      'NeutronInterfaces',
@@ -96,13 +97,13 @@ RESOURCES_CLASSES = ['CinderSnapshots',
 
 def retry(service_name):
     def factory(func):
-        """Decorator allowing to retry in case of failure"""
+        """Decorator allowing to retry in case of failure."""
         def wrapper(*args, **kwargs):
             n = 0
             while True:
                 try:
                     return func(*args, **kwargs)
-                except Exception as e:
+                except Exception:
                     if n == RETRIES:
                         raise DeletionFailed(service_name)
                     n += 1
@@ -117,9 +118,9 @@ def retry(service_name):
 # Classes
 class Session(object):
 
-    """
-    A Session stores information that can be used by the different
-    Openstack Clients. The most important data is:
+    """A Session stores information that can be used by the different Openstack Clients.
+
+    The most important data is:
     * self.token - The Openstack token to be used accross services;
     * self.catalog - Allowing to retrieve services' endpoints.
     """
@@ -147,16 +148,14 @@ class Session(object):
     def get_endpoint(self, service_type):
         try:
             return self.catalog[service_type][0][self.endpoint_type]
-        except KeyError:
+        except (KeyError, IndexError):
             # Endpoint could not be found
             raise EndpointNotFound(service_type)
 
 
 class Resources(object):
 
-    """
-    Abstract base class for all resources to be removed.
-    """
+    """Abstract base class for all resources to be removed."""
 
     def __init__(self, session):
         self.session = session
@@ -165,13 +164,11 @@ class Resources(object):
         pass
 
     def delete(self, resource):
-        """
-        Displays informational message about a resource deletion.
-        """
+        """Displays informational message about a resource deletion."""
         logging.info("* Deleting {}.".format(self.resource_str(resource)))
 
     def purge(self):
-        "Delete all resources."
+        """Delete all resources."""
         # Purging is displayed and done only if self.list succeeds
         resources = self.list()
         c_name = self.__class__.__name__
@@ -180,14 +177,14 @@ class Resources(object):
             retry(c_name)(self.delete)(resource)
 
     def dump(self):
-        "Display all available resources."
+        """Display all available resources."""
         # Resources type and resources are displayed only if self.list succeeds
         resources = self.list()
         c_name = self.__class__.__name__
-        print "* Resources type: {}".format(c_name)
+        print("* Resources type: {}".format(c_name))
         for resource in resources:
-            print self.resource_str(resource)
-        print ""
+            print(self.resource_str(resource))
+        print("")
 
 
 class SwiftResources(Resources):
@@ -257,7 +254,7 @@ class CinderSnapshots(CinderResources):
         return self.client.volume_snapshots.list()
 
     def delete(self, snap):
-        super(CinderResources, self).delete(snap)
+        super(CinderSnapshots, self).delete(snap)
         self.client.volume_snapshots.delete(snap)
 
     def resource_str(self, snap):
@@ -270,7 +267,7 @@ class CinderVolumes(CinderResources):
         return self.client.volumes.list()
 
     def delete(self, vol):
-        """Snapshots created from the volume must be deleted first"""
+        """Snapshots created from the volume must be deleted first."""
         super(CinderVolumes, self).delete(vol)
         self.client.volumes.delete(vol)
 
@@ -278,6 +275,19 @@ class CinderVolumes(CinderResources):
         return "volume {} (id {})".format(vol.display_name, vol.id)
 
 
+class CinderBackups(CinderResources):
+
+    def list(self):
+        return self.client.backups.list()
+
+    def delete(self, backup):
+        super(CinderBackups, self).delete(backup)
+        self.client.backups.delete(backup)
+
+    def resource_str(self, backup):
+        return "backup {} (id {}) of volume {}".format(backup.name, backup.id, backup.volume_id)
+
+
 class NeutronResources(Resources):
 
     def __init__(self, session):
@@ -304,7 +314,7 @@ class NeutronRouters(NeutronResources):
         return self.list_routers()
 
     def delete(self, router):
-        """interfaces must be deleted first"""
+        """Interfaces must be deleted first."""
         super(NeutronRouters, self).delete(router)
         # Remove router gateway prior to remove the router itself
         self.client.remove_gateway_router(router['id'])
@@ -329,7 +339,8 @@ class NeutronInterfaces(NeutronResources):
                                             {'port_id': interface['id']})
 
     def resource_str(self, interface):
-        return "interfaces {} (id)".format(interface['id'])
+        return "interface {} (id {})".format(interface['name'],
+                                             interface['id'])
 
 
 class NeutronPorts(NeutronResources):
@@ -358,7 +369,8 @@ class NeutronNetworks(NeutronResources):
                       self.client.list_networks()['networks'])
 
     def delete(self, net):
-        """
+        """Delete a Neutron network
+
         Interfaces connected to the network must be deleted first.
         Implying there must not be any VM on the network.
         """
@@ -387,7 +399,7 @@ class NeutronSecgroups(NeutronResources):
             raise
 
     def delete(self, secgroup):
-        """VMs using the security group should be deleted first"""
+        """VMs using the security group should be deleted first."""
         super(NeutronSecgroups, self).delete(secgroup)
         self.client.delete_security_group(secgroup['id'])
 
@@ -485,7 +497,7 @@ class CeilometerAlarms(Resources):
 
 class KeystoneManager(object):
 
-    """Manages Keystone queries"""
+    """Manages Keystone queries."""
 
     def __init__(self, username, password, project, auth_url, insecure, **kwargs):
         self.client = keystone_client.Client(
@@ -493,9 +505,11 @@ class KeystoneManager(object):
             tenant_name=project, auth_url=auth_url,
             insecure=insecure, **kwargs)
         self.admin_role_id = None
+        self.tenant_info = None
 
     def get_project_id(self, project_name_or_id=None):
-        """
+        """Get a project by its id
+
         Returns:
         * ID of current project if called without parameter,
         * ID of project given as parameter if one is given.
@@ -505,7 +519,7 @@ class KeystoneManager(object):
             return self.client.tenant_id
 
         try:
-            self.client.tenants.get(project_name_or_id)
+            self.tenant_info = self.client.tenants.get(project_name_or_id)
             # If it doesn't raise an 404, project_name_or_id is
             # already the project's id
             project_id = project_name_or_id
@@ -517,8 +531,19 @@ class KeystoneManager(object):
                     lambda x: x.name == project_name_or_id, tenants)[0].id
             except IndexError:
                 raise NoSuchProject(project_name_or_id)
+
+        if not self.tenant_info:
+            self.tenant_info = self.client.tenants.get(project_id)
         return project_id
 
+    def enable_project(self, project_id):
+        logging.info("* Enabling project {}.".format(project_id))
+        self.tenant_info = self.client.tenants.update(project_id, enabled=True)
+
+    def disable_project(self, project_id):
+        logging.info("* Disabling project {}.".format(project_id))
+        self.tenant_info = self.client.tenants.update(project_id, enabled=False)
+
     def get_admin_role_id(self):
         if not self.admin_role_id:
             roles = self.client.roles.list()
@@ -549,8 +574,8 @@ class KeystoneManager(object):
 def perform_on_project(admin_name, password, project, auth_url,
                        endpoint_type='publicURL', region_name=None,
                        action='dump', insecure=False):
-    """
-    Perform provided action on all resources of project.
+    """Perform provided action on all resources of project.
+
     action can be: 'purge' or 'dump'
     """
     session = Session(admin_name, password, project, auth_url,
@@ -671,10 +696,11 @@ def main():
                                            args.admin_project, args.auth_url,
                                            args.insecure, region_name=args.region_name)
     except api_exceptions.Unauthorized as exc:
-        print "Authentication failed: {}".format(str(exc))
+        print("Authentication failed: {}".format(str(exc)))
         sys.exit(AUTHENTICATION_FAILED_ERROR_CODE)
 
     remove_admin_role_after_purge = False
+    disable_project_after_purge = False
     try:
         cleanup_project_id = keystone_manager.get_project_id(
             args.cleanup_project)
@@ -686,11 +712,19 @@ def main():
                 pass
             else:
                 remove_admin_role_after_purge = True
+
+            # If the project was enabled before the purge, do not disable it after the purge
+            disable_project_after_purge = not keystone_manager.tenant_info.enabled
+            if disable_project_after_purge:
+                # The project is currently disabled so we need to enable it
+                # in order to delete resources of the project
+                keystone_manager.enable_project(cleanup_project_id)
+
     except api_exceptions.Forbidden as exc:
-        print "Not authorized: {}".format(str(exc))
+        print("Not authorized: {}".format(str(exc)))
         sys.exit(NOT_AUTHORIZED)
     except NoSuchProject as exc:
-        print "Project {} doesn't exist".format(str(exc))
+        print("Project {} doesn't exist".format(str(exc)))
         sys.exit(NoSuchProject.ERROR_CODE)
 
     # Proper cleanup
@@ -699,18 +733,22 @@ def main():
         perform_on_project(args.username, args.password, cleanup_project_id,
                            args.auth_url, args.endpoint_type, args.region_name,
                            action, args.insecure)
-    except ConnectionError as exc:
-        print "Connection error: {}".format(str(exc))
+    except requests.exceptions.ConnectionError as exc:
+        print("Connection error: {}".format(str(exc)))
         sys.exit(CONNECTION_ERROR_CODE)
     except (DeletionFailed, InvalidEndpoint) as exc:
-        print "Deletion of {} failed".format(str(exc))
-        print "*Warning* Some resources may not have been cleaned up"
+        print("Deletion of {} failed".format(str(exc)))
+        print("*Warning* Some resources may not have been cleaned up")
         sys.exit(DeletionFailed.ERROR_CODE)
 
     if (not args.dry_run) and (not args.dont_delete_project) and (not args.own_project):
         keystone_manager.delete_project(cleanup_project_id)
     else:
-        # Project is not deleted, we may want to remove ourself from the purged project
+        # Project is not deleted, we may want to disable the project
+        # this must happen before we remove the admin role
+        if disable_project_after_purge:
+            keystone_manager.disable_project(cleanup_project_id)
+        # We may also want to remove ourself from the purged project
         if remove_admin_role_after_purge:
             keystone_manager.undo_become_project_admin(cleanup_project_id)
     sys.exit(0)
